"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const nodegit_1 = require("nodegit");
const base_command_1 = require("../base-command");
const jira_client_1 = require("../jira-client");
class StartWorkCommand extends base_command_1.default {
    async run() {
        const { args } = this.parse(StartWorkCommand);
        const ticketId = args.ticket;
        const config = await this.getConfig();
        this.jiraClient = new jira_client_1.JiraClient({ host: config.jiraHost, username: config.jiraEmail, accessToken: config.jiraAccessToken });
        // Transition ticket to In Progress
        await this.transitionTicketToInProgress(ticketId);
        // Check out new git branch
        const jiraTicket = await this.jiraClient.getJiraTicket(ticketId);
        await this.checkoutBranch(jiraTicket);
    }
    async transitionTicketToInProgress(ticketId) {
        if (!this.jiraClient) {
            this.error('Jira client is null, unable to transition ticket!');
        }
        cli_ux_1.cli.action.start('Transitioning Jira ticket');
        let transitions = await this.jiraClient.getTransitions(ticketId);
        transitions = transitions.filter((transition) => transition.name === 'Start Work');
        if (transitions.length === 0) {
            this.error('Unable to transition this ticket into In Progress!');
        }
        const transition = transitions[0];
        await this.jiraClient.transitionTicket(ticketId, transition.id);
        cli_ux_1.cli.action.stop('done');
    }
    async checkoutBranch(jiraTicket) {
        const defaultBranchName = this.getBranchName(jiraTicket);
        const branchName = await this.promptInput({ message: 'Branch name', default: defaultBranchName });
        cli_ux_1.cli.action.start('Checking out new branch');
        const repo = await nodegit_1.Repository.open('.');
        const commit = await repo.getHeadCommit();
        const branchRef = await repo.createBranch(branchName, commit);
        await repo.checkoutRef(branchRef);
        cli_ux_1.cli.action.stop('done');
        this.log(`Switched to new branch '${branchName}'`);
    }
    getBranchName(jiraTicket) {
        const issueType = jiraTicket.fields.issuetype.name.toUpperCase();
        let prefix;
        switch (issueType) {
            case 'TASK':
                prefix = 'chore/';
                break;
            case 'IMPROVEMENT':
                prefix = 'feature/';
                break;
            case 'BUG':
                prefix = 'bugfix/';
                break;
            default:
                prefix = '';
        }
        const ticketId = jiraTicket.key.toLowerCase();
        const ticketSummary = jiraTicket.fields.summary.toLowerCase();
        const branchName = this.sanitizeBranchName(`${ticketId}_${ticketSummary}`);
        return `${prefix}${branchName}`;
    }
    sanitizeBranchName(name) {
        return name.replace(/'/g, '').replace(/:/g, ' ').replace(/\s/g, '_');
    }
}
StartWorkCommand.description = 'start work on a Jira ticket';
StartWorkCommand.flags = {
    help: command_1.flags.help({ char: 'h' })
};
StartWorkCommand.args = [
    { name: 'ticket', description: 'Jira ticket ID (or key)', required: true }
];
StartWorkCommand.examples = [
    'start ntv-123',
    'start 5678'
];
exports.default = StartWorkCommand;
